@@ -23,7 +23,7 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/config/makedepend/main.c,v 3.35 2006/01/09 14:56:15 dawes Exp $ */
+/* $XFree86: xc/config/makedepend/main.c,v 3.38 2007/06/17 20:24:59 tsi Exp $ */
 
 #include "def.h"
 #ifdef hpux
@@ -64,7 +64,7 @@ int	_debugmask;
 #define DASH_INC_PRE    "#include \""
 #define DASH_INC_POST   "\""
 
-char *ProgramName;
+static char *ProgramName;
 
 char	*directives[] = {
 	"if",
@@ -113,6 +113,7 @@ boolean		show_where_not = FALSE;
 /* Warn on multiple includes of same file */
 boolean		warn_multiple = FALSE;
 
+static char *base_name(char *file);
 static void setfile_cmdinc(struct filepointer *filep, long count, char **list);
 static void redirect(char *line, char *makefile);
 
@@ -139,7 +140,7 @@ catch (int sig)
 #define sa_mask sv_mask
 #define sa_flags sv_flags
 #endif
-struct sigaction sig_act;
+static struct sigaction sig_act;
 #endif /* USGISH */
 
 int
@@ -229,6 +230,7 @@ main(int argc, char *argv[])
 			endmarker = &argv[0][2];
 			if (endmarker[0] == '\0') endmarker = "--";
 			break;
+
 		case 'D':
 			if (argv[0][2] == '\0') {
 				argv++;
@@ -241,6 +243,7 @@ main(int argc, char *argv[])
 				}
 			define(argv[0] + 2, &maininclist);
 			break;
+
 		case 'I':
 			if (incp >= includedirs + MAXDIRS)
 			    fatalerr("Too many -I flags.\n");
@@ -250,6 +253,7 @@ main(int argc, char *argv[])
 				argc--;
 			}
 			break;
+
 		case 'U':
 			/* Undef's override all -D's so save them up */
 			numundefs++;
@@ -264,15 +268,18 @@ main(int argc, char *argv[])
 			}
 			undeflist[numundefs - 1] = argv[0] + 2;
 			break;
+
 		case 'Y':
 			defincdir = argv[0]+2;
 			break;
+
 		/* do not use if endmarker processing */
 		case 'a':
 			if (endmarker) break;
 			if (argv[0][2]) goto badopt;
 			append = TRUE;
 			break;
+
 		case 'w':
 			if (endmarker) break;
 			if (argv[0][2] == '\0') {
@@ -283,6 +290,7 @@ main(int argc, char *argv[])
 			} else
 				width = atoi(argv[0]+2);
 			break;
+
 		case 'o':
 			if (endmarker) break;
 			if (argv[0][2] == '\0') {
@@ -292,6 +300,7 @@ main(int argc, char *argv[])
 			} else
 				objsuffix = argv[0]+2;
 			break;
+
 		case 'p':
 			if (endmarker) break;
 			if (argv[0][2] == '\0') {
@@ -301,6 +310,7 @@ main(int argc, char *argv[])
 			} else
 				objprefix = argv[0]+2;
 			break;
+
 		case 'v':
 			if (endmarker) break;
 			verbose = TRUE;
@@ -309,6 +319,7 @@ main(int argc, char *argv[])
 				_debugmask = atoi(argv[0]+2);
 #endif
 			break;
+
 		case 's':
 			if (endmarker) break;
 			startat = argv[0]+2;
@@ -320,6 +331,7 @@ main(int argc, char *argv[])
 				fatalerr("-s flag's value should start %s\n",
 					"with '#'.");
 			break;
+
 		case 'f':
 			if (endmarker) break;
 			makefile = argv[0]+2;
@@ -335,12 +347,11 @@ main(int argc, char *argv[])
 			warn_multiple = TRUE;
 			break;
 
-		/* Ignore -O, -g so we can just pass ${CFLAGS} to
-		   makedepend
-		 */
+		/* Ignore -O, -g so we can just pass ${CFLAGS} to makedepend */
 		case 'O':
 		case 'g':
 			break;
+
 		case 'i':
 			if (strcmp(&argv[0][1],"include") == 0) {
 				char *buf;
@@ -363,8 +374,10 @@ main(int argc, char *argv[])
 				break;
 			}
 			/* intentional fall through */
+
 		default:
 			if (endmarker) break;
+
 		badopt:
 	/*		fatalerr("unknown option = %s\n", argv[0]); */
 			warning("ignoring option %s\n", argv[0]);
@@ -579,8 +592,13 @@ freefile(struct filepointer *fp)
 
 char *copy(char *str)
 {
-	char	*p = (char *)malloc(strlen(str) + 1);
+	char	*p;
 
+	if (!str)
+		return NULL;
+	p = (char *)malloc(strlen(str) + 1);
+	if (!p)
+		fatalerr("out of memory");
 	strcpy(p, str);
 	return(p);
 }
@@ -704,6 +722,12 @@ char *getnextline(struct filepointer *filep)
 				/* punt lines with just # (yacc generated) */
 				for (cp = bol+1;
 				     *cp && (*cp == ' ' || *cp == '\t'); cp++);
+				/*
+				 * TurboC seems to want its pragmas in column
+				 * 2.
+				 */
+				if (!strncmp(cp, "pragma", 6))
+					whitespace = FALSE;
 				if (*cp) goto done;
 				--p;
 			}
@@ -731,7 +755,7 @@ done:
  * Strip the file name down to what we want to see in the Makefile.
  * It will have objprefix and objsuffix around it.
  */
-char *base_name(char *file)
+static char *base_name(char *file)
 {
 	char	*p;
 
