@@ -1,4 +1,4 @@
-/* $XFree86: xc/lib/X11/XKBMisc.c,v 3.8 2007/04/09 15:37:11 tsi Exp $ */
+/* $XFree86: xc/lib/X11/XKBMisc.c,v 3.9 2007/09/16 03:44:17 tsi Exp $ */
 /************************************************************
 Copyright (c) 1993 by Silicon Graphics Computer Systems, Inc.
 
@@ -659,7 +659,7 @@ register int	i;
 int		width,nOldGroups,oldWidth,newTypes[XkbNumKbdGroups];
 
     if ((!xkb) || (!XkbKeycodeInRange(xkb,key)) || (!xkb->map) ||
-	(!xkb->map->types)||(!newTypes)||((groups&XkbAllGroupsMask)==0)||
+	(!xkb->map->types)||((groups&XkbAllGroupsMask)==0)||
 	(nGroups>XkbNumKbdGroups)) {
 	return BadMatch;
     }
