@@ -1,4 +1,3 @@
-/* $Xorg: TMaction.c,v 1.5 2001/02/09 02:03:58 xorgcvs Exp $ */
 /*LINTLIBRARY*/
 
 /***********************************************************
@@ -59,7 +58,7 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/lib/Xt/TMaction.c,v 3.10 2004/05/26 22:24:40 dickey Exp $ */
+/* $XFree86: xc/lib/Xt/TMaction.c,v 3.12 2006/01/09 14:59:22 dawes Exp $ */
 
 /* TMaction.c -- maintains the state table of actions for the translation
  *              manager.
@@ -630,6 +629,7 @@ void _XtBindActions(
 		int			localUnbound;
 		TMBindCacheStatusRec	bcStatusRec;
 
+		bcStatusRec.refCount = 0;
 		procs = (XtActionProc *)
 		  XtStackAlloc(stateTree->numQuarks * sizeof(XtActionProc),
 			       stackProcs);
