@@ -1,4 +1,4 @@
-/* $XFree86: xc/lib/dmx/dmx.c,v 1.1 2004/06/30 20:21:37 martin Exp $ */
+/* $XFree86: xc/lib/dmx/dmx.c,v 1.2 2005/10/14 15:16:00 tsi Exp $ */
 /*
  * Copyright 2002-2004 Red Hat Inc., Durham, North Carolina.
  *
@@ -41,11 +41,11 @@
 
 #define NEED_REPLIES
 #include <X11/Xlibint.h>
-#include "Xext.h"
+#include <X11/extensions/Xext.h>
 #define EXTENSION_PROC_ARGS void *
-#include "extutil.h"
-#include "dmxproto.h"
-#include "dmxext.h"
+#include <X11/extensions/extutil.h>
+#include <X11/extensions/dmxproto.h>
+#include <X11/extensions/dmxext.h>
 
 static XExtensionInfo dmx_extension_info_data;
 static XExtensionInfo *dmx_extension_info = &dmx_extension_info_data;
