@@ -19,7 +19,7 @@ INCIDENTAL OR CONSEQUENTIAL DAMAGES, ARISING OUT OF OR IN ANY WAY CONNECTED
 WITH THE SPEEDO SOFTWARE OR THE BITSTREAM CHARTER OUTLINE FONT.
 
 */
-/* $XFree86: xc/lib/font/Speedo/set_spcs.c,v 1.5 2006/01/09 14:59:27 dawes Exp $ */
+/* $XFree86: xc/lib/font/Speedo/set_spcs.c,v 1.6 2006/12/10 15:58:24 tsi Exp $ */
 
 
 /*************************** S E T _ S P C S . C *****************************
@@ -31,11 +31,13 @@ WITH THE SPEEDO SOFTWARE OR THE BITSTREAM CHARTER OUTLINE FONT.
 #include "spdo_prv.h"               /* General definitions for Speedo    */
 #include "keys.h"
 
-#define   DEBUG      0
+#ifndef DEBUG
+#define DEBUG 0
+#endif
 
 #if DEBUG
 #include <stdio.h>
-#define SHOW(X) printf("X = %d\n", X)
+#define SHOW(X) printf("X = %lx\n", (unsigned long)X)
 #else
 #define SHOW(X)
 #endif
